#pragma once

#include <ext/memory>

#include <abstraction/AnyaryOperationAbstraction.hpp>
#include <abstraction/ValueOperationAbstraction.hpp>

#include <factory/NormalizeFactory.hpp>

namespace abstraction {

template < class ParamType >
class SetAbstraction : virtual public AnyaryOperationAbstraction < ParamType >, virtual public ValueOperationAbstraction < ext::set < ParamType > > {
public:
	std::shared_ptr < abstraction::Value > run ( ) const override {
		ext::set < ParamType > theSet;
		for ( const std::shared_ptr < abstraction::Value > & param : this->getParams ( ) ) {
			theSet.insert ( abstraction::retrieveValue < ParamType > ( param ) );
		}

		object::Object normalized = factory::NormalizeFactory::normalize < std::decay_t < ext::set < ParamType > > > ( std::move ( theSet ) );
		return std::make_shared < abstraction::ValueHolder < object::Object > > ( std::move ( normalized ), true );
	}

};

} /* namespace abstraction */

